/*
 * tfcx.cc -- Detects specified planet or ship FCs and changes them to random numbers
 *
 * Copyright (C) 2000 Heiko Schlenker <hschlen@gmx.de>
 *
 * Author: Heiko Schlenker <hschlen@gmx.de>
 * Version: $Revision: 1.7 $
 * Status: quick & dirty
 *
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <tkhost.h>
#include <tkiocomm.h>

#define MAXINBUFSIZE 8

static const char rcsid[] = "$Id: tfcx.cc,v 1.7 2000-07-31 18:36:50+02 hschlen Exp hschlen $";
const char TFCX_VERSION[] = "$Revision: 1.7 $";
const char ReleaseDate[] = "$Date: 2000-07-31 18:36:50+02 $";
const char Notice[] = "Copyright (C) 2000 Heiko Schlenker.\r\n\
Tfcx comes with ABSOLUTELY NO WARRANTY.\r\n\
This is free software. See COPYING for details.";


int
main(int argc, char *argv[])
{
    int i = 0, j = 0;
    FILE *gLogFile = NULL;
    FILE *InFile = NULL;
    int line = 0;
    int returncode = 0;
    char FC[4] = {'\0'};
    char inbuf[MAXINBUFSIZE] = {'\0'};
    PLANETDATA *planpad = NULL;
    SHIPDATA *shippad = NULL;


    /* Parse our command-line parameters. First parameter is the game directory. */
    switch (argc) {
    case 2:
	break;
	/* FALL THROUGH */

    case 1:
	/* break; */

    default:
	printf("tfcx - Detects specified planet or ship FCs and changes them to random numbers.\r\n");
	printf("%s (%s)\r\n", TFCX_VERSION, ReleaseDate);
	printf("%s\r\n\r\n", Notice);
	printf("Usage: tfcx [GameDirectory]\r\n\r\n");
	exit(EXIT_FAILURE);
    }

    set_game_dir(argv[1]);

    gLogFile = mkopen("tfcx.log",false,'o');
    if (!gLogFile) {
	fprintf(stderr, "Error: Cannot open \"tfcx.log\"!\r\n");
	returncode = EXIT_FAILURE;
	goto error;
    }

    fprintf(gLogFile, "%s\r\n\r\n", rcsid);

    InFile = mkopen("tfcx.ini",false,'r');
    if (!InFile) {
	fprintf(gLogFile, "Error: Cannot open input file \"%s\"\r\n", "tfcx.ini");
	fprintf(stderr, "Error: Cannot open input file \"%s\"\r\n", "tfcx.ini");
	returncode = EXIT_FAILURE;
	goto error;
    }

    planpad = read_all_planetdata();
    if (!planpad) {
	fprintf(gLogFile,"Error: Couldn't open pdata.hst\r\n");
	fprintf(stderr,"Error: Couldn't open pdata.hst\r\n");
	returncode = EXIT_FAILURE;
	goto error;
    }

    shippad = read_all_shipdata();
    if (!shippad) {
	fprintf(gLogFile,"Error: Couldn't open ships.hst\r\n");
	fprintf(stderr,"Error: Couldn't open ships.hst\r\n");
	returncode = EXIT_FAILURE;
	goto error;
    }

    i = time(NULL);
    srand(i | 1);   /* set random seed */

    printf("\r\n");

    /* now scan through planets and ships and see which has a certain FC code */
    while (!feof(InFile)) {
	line++;
	if (0 == fgets(inbuf, 4, InFile)) {
	    if (!feof(InFile)) {
		fprintf(stderr,"Error: Error reading input file \"%s\" at line %u!\r\n", "tfcx.ini", line);
		fprintf(gLogFile, "Error: Error reading input file \"%s\" at line %u!\r\n", "tfcx.ini", line);
		returncode = EXIT_FAILURE;
		goto error;
	    }
	    break;
	}
	/* remove newline or carriage return */
	for (i = 0; i < MAXINBUFSIZE; i++) {
	    if (('\n' == inbuf[i]) || ('\r' == inbuf[i])) {
		inbuf[i] = '\0';
	    }
	}
	if (inbuf[0] != '\0') {
	    for (i = 1; i <= NUM_PLANETS; i++) {
		if ((strncmp(inbuf, planpad[i].sCode, strlen(inbuf)) == 0)) {
		    /* scans first chars of the FC and compares them */
		    j = 1 + (int)(999.0*rand()/(RAND_MAX+1.0));  /* return 3 digit number */
		    if (j < 100) {
			j += 100;  /* so we don't have to worry about zeros */
		    }
		    /* stuffs j into a string - there is no good C function for this */
		    sprintf(FC, "%u", j);
		    planpad[i].sCode = FC;
		    if (!write_planetdata(i, &planpad[i])) {
			fprintf(gLogFile, "Error: Couldn't write data of planet %d\r\n!", i);
			fprintf(stderr, "Error: Couldn't write data of planet %d\r\n!", i);
			returncode = EXIT_FAILURE;
			goto error;
		    }
		    fprintf(gLogFile, "Planet number %u has a \"%s\" setting!  Changed!\r\n", i, inbuf);
		    printf("Planet number %u has a \"%s\" setting!  Changed!\r\n", i, inbuf);
		}
	    }
	    for (i = 1; i <= MAXSHIP; i++) {
		if ((strncmp(inbuf, shippad[i].sCode, strlen(inbuf)) == 0)) {
		    /* scans first chars of the FC and compares them */
		    j = 1 + (int)(999.0*rand()/(RAND_MAX+1.0));  /* return 3 digit number */
		    if (j < 100) {
			j += 100;  /* so we don't have to worry about zeros */
		    }
		    /* stuffs j into a string - there is no good C function for this */
		    sprintf(FC, "%u", j);
		    shippad[i].sCode = FC;
		    if (!write_shipdata(i, &shippad[i])) {
			fprintf(gLogFile, "Error: Couldn't write data of ship %d\r\n!", i);
			fprintf(stderr, "Error: Couldn't write data of ship %d\r\n!", i);
			returncode = EXIT_FAILURE;
			goto error;
		    }
		    fprintf(gLogFile, "Ship number %u has a \"%s\" setting!  Changed!\r\n", i, inbuf);
		    printf("Ship number %u has a \"%s\" setting!  Changed!\r\n", i, inbuf);
		}
	    }
	}
    }
    printf("\r\n");

    /* Close down. */
error:

    if (InFile) {
	fclose(InFile);
    }
    if (gLogFile) {
	fclose(gLogFile);
    }

    delete[] planpad;
    delete[] shippad;

    exit(returncode);
}
